# -*- coding: utf-8 -*-
"""Sample Configuration
"""

# For Maverick
site_prefix = "/"
template = "Galileo"
index_page_size = 10
archives_page_size = 30
fetch_remote_imgs = False
enable_jsdelivr = {
    "enabled": False,
    "repo": "AlanDecode/Maverick@gh-pages"
}
locale = "Asia/Shanghai"
category_by_folder = False

# For site
site_name = "repo-cli with Maverick"
site_logo = "${static_prefix}android-chrome-512x512.png"
site_build_date = "2019-12-06T12:00+08:00"
author = "AlanDecode"
email = "hi@imalan.cn"
author_homepage = "https://www.imalan.cn"
description = "make by repo-cli, power by Maverick , Theme Galileo."
key_words = ["Maverick", "AlanDecode", "Galileo", "blog"]
language = 'english'
external_links = [
    {
        "name": "Maverick",
        "url": "https://github.com/AlanDecode/Maverick",
        "brief": "🏄‍ Go My Own Way."
    },
    {
        "name": "Triple NULL",
        "url": "https://www.imalan.cn",
        "brief": "Home page for AlanDecode."
    }
]
nav = [
    {
        "name": "Home",
        "url": "${site_prefix}",
        "target": "_self"
    },
    {
        "name": "Archives",
        "url": "${site_prefix}archives/",
        "target": "_self"
    },
    {
        "name": "About",
        "url": "${site_prefix}about/",
        "target": "_self"
    }
]

social_links = [
    {
        "name": "Twitter",
        "url": "https://twitter.com/AlanDecode",
        "icon": "gi gi-twitter"
    },
    {
        "name": "GitHub",
        "url": "https://github.com/AlanDecode",
        "icon": "gi gi-github"
    },
    {
        "name": "Weibo",
        "url": "https://weibo.com/5245109677/",
        "icon": "gi gi-weibo"
    }
]

valine = {
    "enable": False,
    "el": '#vcomments',
    "appId": "IKRAfuPq0zrz6Wfje8ahHAIP-gzGzoHsz",
    "appKey": "lFaCWkd4xCs0Ng5UWs1eHNwU",
    "visitor": True,
    "recordIP": True
}

head_addon = ''

footer_addon = ''

body_addon = ''



# -*- coding: utf-8 -*-
"""Builder

1. search source_dir recursively
2. build website structure
3. render pages
4. write to disk
"""

import os
import sys
import shutil
import functools
import pathlib

from .Utils import logged_func, print_color, Color, unify_joinpath, force_rmtree, run
from .Content import Content, ContentList
from .Config import Config
from . import Galileo


class TemplateError(BaseException):
    pass


class Builder:
    def __init__(self, conf: Config):
        self._config = conf
        self._posts = ContentList()
        self._pages = ContentList()

    @logged_func('')
    def clean(self):
        if os.path.exists(self._config.build_dir):
            try:
                shutil.rmtree(self._config.build_dir)
            except BaseException as e:
                print(e)

    @logged_func()
    def setup_theme(self):
        """Setup theme in this method.

        1. handle sys.path for local theme
        2. clone from remote for git theme
        3. install deps for theme
        """
        # template = ''
        # template_dep_file = ''
        # mvrk_path = self._config.mvrk_path

        # def clone_remote_theme(config: dict):
        #     repo_dir = os.path.abspath(mvrk_path + '/Templates/' + config['name'])
        #     if os.path.exists(repo_dir):
        #         force_rmtree(repo_dir)
        #
        #     repo_url = config['url']
        #     repo_branch = config.get('branch', 'master')
        #     repo_tag = config.get('tag', '')
        #
        #     def safe_run(command, cwd):
        #         try:
        #             run(command, cwd)
        #         except:
        #             raise TemplateError('Cannot fetch theme from '+repo_url)
        #
        #     safe_run('git clone -b %s %s %s' %
        #              (repo_branch, repo_url, repo_dir), mvrk_path)
        #     if repo_tag != '':
        #         safe_run('git checkout %s' & repo_tag, repo_dir)
        #
        # # 根据不同配置情况获取主题文件
        # if type(self._config.template) == str:
        #     # 如果主题配置项是一个字符串，则用户选用的是默认主题
        #     built_in_themes = ['Kepler', 'Galileo']
        #     if not os.path.exists(unify_joinpath(mvrk_path + '/Templates', self._config.template)):
        #         # 如果没有相应的主题文件
        #         if self._config.template in built_in_themes:
        #             # 如果主题是内置主题的一种，通过仓库获取主题
        #             clone_remote_theme({
        #                 "name": self._config.template,
        #                 "type": "git",
        #                 "url": "https://github.com/AlanDecode/Maverick-Theme-%s.git" % self._config.template,
        #                 "branch": "latest"
        #             })
        #             template = '.'.join(['Templates', self._config.template])
        #             template_dep_file = mvrk_path + '/Templates/%s/requirements.txt' % self._config.template
        #         else:
        #             # 否则，提示没有此主题
        #             raise TemplateError(
        #                 'Can not found local theme '+self._config.template)
        #     else:
        #         # TODO 如果是用户自己创建了一个同名文件
        #         # 如果存在响应的主题文件
        #         template = '.'.join(['Templates', self._config.template])
        #         template_dep_file = mvrk_path + '/Templates/%s/requirements.txt' % self._config.template
        #
        # elif type(self._config.template) == dict:
        #     # 如果主题配置项是一个字典，则根据配置获取主题文件
        #     if self._config.template['type'] == 'local':
        #         local_dir = os.path.abspath(self._config.template['path'])
        #         if not os.path.exists(local_dir):
        #             raise TemplateError(
        #                 'Can not found local theme '+self._config.template['name'])
        #         else:
        #             sys.path.insert(0, os.path.dirname(local_dir))
        #             template = self._config.template['name']
        #             template_dep_file = unify_joinpath(
        #                 local_dir, 'requirements.txt')
        #     elif self._config.template['type'] == 'git':
        #         clone_remote_theme(self._config.template)
        #         template = '.'.join(
        #             ['Templates', self._config.template['name']])
        #         template_dep_file = mvrk_path + '/Templates/%s/requirements.txt' % self._config.template['name']
        #
        # else:
        #     raise TemplateError('Invalid template config',
        #                         str(self._config.template))
        #
        # # handle deps for theme
        # if os.path.exists(template_dep_file) and os.path.isfile(template_dep_file):
        #     try:
        #         run('pip install -r %s' % template_dep_file, '.')
        #     except:
        #         raise TemplateError('Can not install dependencies for theme.')

        # 导入主题到运行环境
        self._template = Galileo

    def build_all(self):
        """Init building
        """

        # delete last build
        self.clean()

        print('Loading contents...')
        # test_src
        # os.walk
        # 生成目录树中的文件名，方式是按上->下或下->上顺序浏览目录树。
        # 对于以 top 为根的目录树中的每个目录（包括 top 本身）
        # 它都会生成一个三元组 (dirpath, dirnames, filenames)。
        # 按照层级关系从上往下迭代目录树，并且在每个层级都会生成三个元组 dirpath dirnames filenames
        # dirpath 存储当前目录层级的路径(相对还是绝对取决你你传入的是相对还是绝对
        # dirnames 存储当前层级存在的目录名
        # filenames 存储当前层级存在的文件名
        walker = os.walk(self._config.source_dir)
        # PurePath 一个通用的类，代表当前系统的路径风格
        source_abs_dir = pathlib.PurePath(
            os.path.abspath(self._config.source_dir))

        for path, _, filelist in walker:
            for file in filelist:
                # 如果是 md 文件
                if file.split(".")[-1].lower() == "md" or file.split(".")[-1].lower() == "markdown":
                        content_path = os.path.abspath(unify_joinpath(path, file))
                        content = Content(content_path)
                        if not content.get_meta("status").lower() in [
                                "publish", "published", "hide", "hidden"]:
                            continue

                        # 区分布局方案
                        layout = content.get_meta("layout").lower()
                        if layout == "post":
                            # 如果是通过文件布局分类，将文件布局情况解析出来并添加到文章源信息里
                            if (self._config.category_by_folder):
                                relative_path = pathlib.PurePath(
                                    content_path).relative_to(source_abs_dir)
                                relative_path = list(relative_path.parts[0:-1])
                                if len(relative_path) == 0:
                                    relative_path.append('Default')
                                content.update_meta(
                                    'categories', relative_path)
                            # 将解析好的文章导入到文章列表内
                            self._posts.append(content)
                        elif layout == "page":
                            # 将解析好的页面导入到页面列表内
                            content.update_meta('categories', [])
                            self._pages.append(content)

        print('Contents loaded.')
        # 通过制定方法重新排列文章顺序
        self._posts = ContentList(sorted(self._posts,
                                         key=functools.cmp_to_key(
                                             Content.cmp_by_date),
                                         reverse=True))
        self._pages = ContentList(sorted(self._pages,
                                         key=functools.cmp_to_key(
                                             Content.cmp_by_date),
                                         reverse=True))
        # 载入主题
        self.setup_theme()
        # 运行主题的解析函数
        self._template.render(self._config, self._posts, self._pages)

        print_color('\nAll done, enjoy.', Color.GREEN.value)



# -*- coding: utf-8 -*-
"""Image
Handle cache
"""

import mistune
import os
from ..Cache import cache_img


class ImageRenderer(mistune.HTMLRenderer):
    md_path = ''
    g_hooks = {}

    def image(self, src, alt="", title=None):
        # cache image, parse its width and height
        image_meta = cache_img(src, os.path.dirname(self.md_path))
        image_meta['title'] = title
        image_meta['alt'] = alt

        def default_image(image):
            figcaption = image['title'] or image['alt'] or ''

            attr = 'data-width="%s" data-height="%s"' % (
                image['width'], image['height'])

            if figcaption != "":
                figcaption = '<figcaption>%s</figcaption>' % figcaption

            return '<figure><img loading="lazy" %s src="%s" alt="%s" />%s</figure>' \
                % (attr, image['src'], image['alt'], figcaption)

        return self.g_hooks.get('output_image', default_image)(image_meta)



# -*- coding: utf-8 -*-
"""Galileo

Default theme for Maverick
"""

from ..Template import Template, Pager
from ..Content import ContentList, group_by_category, group_by_tagname
from ..Utils import logged_func, gen_hash, unify_joinpath, copytree
from ..Utils import safe_write, safe_read
from .utils import tr, build_navs, build_links, filterPrefix

import os
import re
import json
import math


def render(conf, posts, pages):
    """Override this
    """
    Galileo(conf, posts, pages)()


class Galileo(Template):
    def render(self):
        self._env.globals['tr'] = tr
        self._env.globals['len'] = len
        self._env.globals['build_navs'] = build_navs
        self._env.globals['build_links'] = build_links
        self._env.globals['get_path'] = filterPrefix

        try:
            from ..Markdown import g_hooks
            g_hooks.add_hook('output_image', self.output_image)
        except BaseException:
            pass

        self.build_search_cache()
        self.gather_meta()
        self.build_posts()
        self.build_pages()

        # filter out hidden posts before building post list
        self._posts = ContentList(
            [item for item in self._posts if not item.skip])
        self._pages = ContentList(
            [item for item in self._pages if not item.skip])

        self.build_index()
        self.build_archives()
        self.build_categories()
        self.build_tags()

        # copy static files to build_dir
        def copy_assets(src, dist):
            source_dir = unify_joinpath(os.path.dirname(__file__), src)
            dist_dir = unify_joinpath(self._config.build_dir, dist)

            if not os.path.exists(source_dir):
                return

            if not os.path.exists(dist_dir):
                os.mkdir(dist_dir)
            copytree(source_dir, dist_dir)
        copy_assets('assets', 'assets')

    def output_image(self, image):
        figcaption = image['title'] or image['alt'] or ''

        src = image['src']

        style = ''
        attr = ''
        if image['width'] != -1 and image['height'] != -1:
            style = 'style="flex: %s"' % str(image['width'] * 50 / image['height'])
        else:
            attr = 'size-undefined'

        if figcaption != "":
            figcaption = '<figcaption>%s</figcaption>' % figcaption

        return '<figure %s %s><img loading="lazy" width="%s" height="%s" src="%s" />%s</figure>' \
            % (style, attr, image['width'], image['height'], src, figcaption)

    def gather_meta(self):
        self._tags = set()
        self._categories = set()
        for content in self._posts:
            meta = content.meta
            self._tags = set(meta["tags"]) | self._tags
            self._categories = set(meta["categories"]) | self._categories

    @logged_func('')
    def build_index(self):
        pager = Pager(self._posts, self._config.index_page_size)
        total_pages = pager.get_total_pages()

        for current_page, current_list in pager:
            _, local_path = self._router.gen("index", "", current_page)
            local_path += "index.html"

            template = self._env.get_template("index.html")
            output = template.render(
                content_list=current_list,
                current_page=current_page,
                max_pages=total_pages)
            safe_write(local_path, output)

    @logged_func('')
    def build_archives(self):
        pager = Pager(self._posts, self._config.archives_page_size)
        total_pages = pager.get_total_pages()

        for current_page, current_list in pager:
            _, local_path = self._router.gen("archives", "", current_page)
            local_path += "index.html"

            template = self._env.get_template("archives.html")
            output = template.render(
                content_list=current_list,
                current_page=current_page,
                max_pages=total_pages)
            safe_write(local_path, output)

    @logged_func('')
    def build_tags(self):
        for tag in self._tags:
            posts = self._posts.re_group(group_by_tagname(tag))

            pager = Pager(posts, self._config.archives_page_size)
            total_pages = pager.get_total_pages()

            for current_page, current_list in pager:
                _, local_path = self._router.gen("tag", tag, current_page)
                local_path += "index.html"

                template = self._env.get_template("tags.html")
                output = template.render(
                    tag_name=tag,
                    content_list=current_list,
                    current_page=current_page,
                    max_pages=total_pages)
                safe_write(local_path, output)

    @logged_func('')
    def build_categories(self):
        for category in self._categories:
            posts = self._posts.re_group(group_by_category(category))

            pager = Pager(posts, self._config.archives_page_size)
            total_pages = pager.get_total_pages()

            for current_page, current_list in pager:
                _, local_path = self._router.gen("category", category, current_page)
                local_path += "index.html"

                template = self._env.get_template("categories.html")
                output = template.render(
                    cate_name=category,
                    content_list=current_list,
                    current_page=current_page,
                    max_pages=total_pages)
                safe_write(local_path, output)

    @logged_func()
    def build_posts(self):
        total_posts = len(self._posts)
        for index in range(total_posts):
            content = self._posts[index]

            # find visible prev and next
            index_next = index
            content_next = None
            while content_next is None and index_next > 0:
                index_next -= 1
                if not self._posts[index_next].skip:
                    content_next = self._posts[index_next]

            index_prev = index
            content_prev = None
            while content_prev is None and index_prev < total_posts-1:
                index_prev += 1
                if not self._posts[index_prev].skip:
                    content_prev = self._posts[index_prev]

            meta = content.meta
            _, local_path = self._router.gen_by_meta(meta)
            local_path += "index.html"

            template = self._env.get_template("post.html")
            output = template.render(
                content=content,
                content_prev=content_prev,
                content_next=content_next
            )
            safe_write(local_path, output)
            print('Finished: ' + content.get_meta('title'))

    @logged_func()
    def build_pages(self):
        total_pages = len(self._pages)
        for index in range(total_pages):
            content = self._pages[index]
            content_next = self._pages[index-1] if index > 0 else None
            content_prev = self._posts[index +
                                       1] if index < total_pages-1 else None

            _, local_path = self._router.gen_by_content(content)
            local_path += "index.html"

            template = self._env.get_template("page.html")
            output = template.render(
                content=content,
                content_prev=content_prev,
                content_next=content_next
            )
            safe_write(local_path, output)
            print('Finished: ' + content.get_meta('title'))

    @logged_func()
    def build_search_cache(self):
        """build search cache json
        """
        def render_search_cache(post_list, page_list):
            router = self._router

            def strip(text):
                r = re.compile(r'<[^>]+>', re.S)
                return r.sub('', text)

            def gen_entry(content):
                entry = {
                    "title": content.get_meta('title'),
                    "date": str(content.get_meta('date')),
                    "path": router.gen_permalink_by_content(content),
                    "text": strip(content.parsed),
                    "categories": [],
                    "tags": []
                }
                if (content.get_meta('layout') == 'post'):
                    for cate in content.get_meta('categories'):
                        entry['categories'].append({
                            "name": cate,
                            "slug": cate,
                            "permalink": router.gen_permalink('category', cate, 1)
                        })
                    for tag in content.get_meta('tags'):
                        entry['tags'].append({
                            "name": tag,
                            "slug": tag,
                            "permalink": router.gen_permalink('tag', tag, 1)
                        })
                return entry

            posts = [gen_entry(post) for post in post_list if not post.skip]
            pages = [gen_entry(page) for page in page_list if not page.skip]

            cache = json.dumps({
                "posts": posts,
                "pages": pages
            })

            return cache

        cache_str = render_search_cache(self._posts, self._pages)
        search_cache_hash = gen_hash(cache_str)
        safe_write(unify_joinpath(
            self._config.build_dir, search_cache_hash + '.json'), cache_str)

        self._env.globals['search_cache_hash'] = search_cache_hash



# -*- coding: utf-8 -*-

import os
import moment
from .Config import g_conf

class Metadata(dict):
    """Metadata

    文章以及页面的元数据
    """

    def __init__(self, fr):
        dict.__init__({})
        self["title"] = str(fr.get("title", ""))
        self["slug"] = str(fr.get("slug", self["title"]))
        self["date"] = moment.date(str(fr.get("date", ""))).locale(g_conf.locale)
        self["layout"] = str(fr.get("layout", "post"))
        self["status"] = str(fr.get("status", "publish"))
        self["author"] = str(fr.get("author", ""))
        self["banner"] = str(fr.get("banner", ""))
        self["excerpt"] = str(fr.get("excerpt", ""))
        self["path"] = ""
        self["showfull"] = bool(fr.get("showfull", False))
        self["comment"] = bool(fr.get("comment", True))

        # 解析包含的 tag（无序）
        self["tags"] = fr.get("tags", []) or []

        # 解析包含的类别（有序）
        self["categories"] = fr.get("categories", []) or []
        if len(self["categories"]) == 0:
            self["categories"].append('Default')



# -*- coding: utf-8 -*-
"""mistune

Custom plugins for mistune.

Author: AlanDecode | 熊猫小A
Link:   https://www.imalan.cn
GitHub: https://github.com/AlanDecode
"""

import mistune
from .ruby import plugin_ruby
from .autotag import plugin_autotag
from .blockmath import plugin_mathblock
from .linkcard import plugin_linkcard
from .inline_footnote import plugin_inlinefootnote
from .image import ImageRenderer
from .blockcode import HighlightRenderer
from .paragraph import ParagraphRenderer


class MyRenderer(ImageRenderer, HighlightRenderer, ParagraphRenderer):
    def __init__(self, escape=True, md_path='', g_hooks=None):
        super(MyRenderer, self).__init__(escape=escape)
        self.md_path = md_path
        self.g_hooks = g_hooks



# -*- coding: utf-8 -*-
"""Template

All templates should inherit this class, and override `render` method
"""

from .Config import Config
from .Utils import filterPlaceholders, unify_joinpath, logged_func
from .Utils import copytree, safe_read, safe_write
from .Router import Router
from .Content import ContentList
from .Cache import dump_log
from . import Maverick_Version

from jinja2 import Environment, FileSystemLoader
from feedgen.feed import FeedGenerator
import moment
import re
import os
import inspect
import json
import shutil
import math
import typing

def render(conf, posts, pages):
    """Override this
    """
    Template(conf, posts, pages)()


class Template:
    def __init__(self, conf: Config, posts: ContentList, pages: ContentList):
        self._config = conf
        self._posts = posts
        self._pages = pages
        self._router = Router(conf)

        # 获取类所在文件的绝对路径 | 目的是获取主题目录下 templates 目录的绝对路径
        path = os.path.abspath(inspect.getfile(self.__class__))
        path = unify_joinpath(os.path.dirname(path), 'templates')
        # 创建 jinja2 对象
        self._env = Environment(loader=FileSystemLoader(path))
        self._env.globals['moment'] = moment
        self._env.globals['config'] = self._config
        self._env.globals['Router'] = Router(self._config)
        self._env.globals['fp'] = filterPlaceholders
        self._env.globals['maverick_version'] = Maverick_Version
        self._env.globals['json'] = json

    def render(self):
        """Override this
        """
        pass

    @logged_func('')
    def _build_feed(self):
        router = Router(self._config)
        fp = filterPlaceholders

        fg = FeedGenerator()
        fg.id(self._config.site_prefix)
        fg.title(self._config.site_name)
        fg.author({
            'name': fp(self._config.author),
            'email': fp(self._config.email)
        })
        fg.link(href=self._config.site_prefix, rel='alternate')
        fg.logo(fp(self._config.site_logo))
        fg.subtitle(fp(self._config.description))
        fg.description(fp(self._config.description) or ' ')
        fg.language(fp(self._config.language))
        fg.lastBuildDate(moment.now().locale(self._config.locale).date)
        fg.pubDate(moment.now().locale(self._config.locale).date)

        for post in self._posts[:10]:
            meta = post.meta
            fe = fg.add_entry()
            fe.title(meta['title'])
            fe.link(href=router.gen_permalink_by_meta(meta))
            fe.guid(router.gen_permalink_by_meta(meta), True)
            fe.pubDate(meta['date'].date)
            fe.author({
                'name': meta['author'],
                'uri': fp(self._config.author_homepage),
                'email': fp(self._config.email)
            })
            fe.content(post.parsed)

        if not os.path.exists(unify_joinpath(self._config.build_dir, 'feed/atom')):
            os.makedirs(unify_joinpath(self._config.build_dir, 'feed/atom'))

        fg.rss_file(unify_joinpath(self._config.build_dir, 'feed/index.xml'))
        fg.rss_file(unify_joinpath(self._config.build_dir, 'feed/index.html'))
        fg.atom_file(unify_joinpath(
            self._config.build_dir, 'feed/atom/index.xml'))
        fg.atom_file(unify_joinpath(
            self._config.build_dir, 'feed/atom/index.html'))

    @logged_func('')
    def _build_sitemap(self):
        template = r'''<?xml version="1.0" encoding="utf-8"?>
<urlset 
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9 http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd"
xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">	
{{output}}
</urlset>
'''
        template_content = r'''
<url>
    <loc>{{loc}}</loc>
    <lastmod>{{lastmod}}</lastmod>
    <changefreq>always</changefreq>
    <priority>{{priority}}</priority>
</url>
'''
        output = ''
        for page in self._pages:
            output += template_content \
                .replace(r'{{loc}}', self._router.gen_permalink_by_content(page)) \
                .replace(r'{{lastmod}}', page.get_meta('date').format('YYYY-MM-DD')) \
                .replace(r'{{priority}}', '0.8')
        for post in self._posts:
            output += template_content \
                .replace(r'{{loc}}', self._router.gen_permalink_by_content(post)) \
                .replace(r'{{lastmod}}', post.get_meta('date').format('YYYY-MM-DD')) \
                .replace(r'{{priority}}', '0.5')
        safe_write(
            unify_joinpath(self._config.build_dir, 'sitemap.xml'),
            template.replace(r'{{output}}', output))

    @logged_func('')
    def _build_static(self):
        # copy files under source_dir/static to build_dir
        static_dir = unify_joinpath(self._config.source_dir, 'static')
        if os.path.isdir(static_dir):
            copytree(static_dir, self._config.build_dir)

        # copy images to build_dir/archives/assets
        dist_dir = unify_joinpath(
            self._config.build_dir, 'archives/assets')
        src_dir = unify_joinpath(self._config.mvrk_path, 'cached_imgs')
        if not os.path.exists(dist_dir):
            os.makedirs(dist_dir)

        tmp_dir = unify_joinpath(self._config.mvrk_path, 'tmp')
        cached_imgs = set(json.loads(
            safe_read(unify_joinpath(tmp_dir, 'used_imgs.json')) or '[]'))
        for img in cached_imgs:
            shutil.copy(unify_joinpath(src_dir, img), dist_dir)

        if os.path.exists(tmp_dir):
            shutil.rmtree(tmp_dir)

    def __call__(self):
        self.render()
        dump_log()
        self._build_static()
        self._build_feed()
        self._build_sitemap()


class Pager:
    def __init__(self, content_list: ContentList, page_size: int):
        self._list = content_list
        self._page_size = page_size
        self._current_page = 0

    def __next__(self):
        self._current_page += 1

        if self._current_page < 1 or self._current_page > self.get_total_pages():
            raise StopIteration

        return self._current_page, self.get_page(self._current_page)

    def __iter__(self):
        self._current_page = 0
        return self

    def get_total_contents(self):
        return len(self._list)

    def get_total_pages(self):
        return math.ceil(len(self._list) / self._page_size)

    def get_page(self, page: int):
        """get contents in `index`-th page
        """
        if page < 1 or page > self.get_total_pages():
            return ContentList()

        return self._list[
            (page-1)*self._page_size : min(page*self._page_size, len(self._list))]



# -*- coding: utf-8 -*-
"""LinkCard
[name](link)+(image)
"""

import re

LINKCARD_PATTERN = (
    r'\[([\s\S]+?)\]\(([\s\S]+?)\)\+\(([\s\S]+?)\)'
)


def parse_linkcard(self, m, state):
    match = re.match(LINKCARD_PATTERN, m.group(0))
    name, link, image = match.group(1), match.group(2), match.group(3)
    return 'linkcard', name, link, image


def render_html_linkcard(name, link, image):
    return '<a style="display:block" target="_blank" href="%s" class="board-item"> \
            <div class="board-thumb"><img src="%s"></div> \
            <div class="board-title">%s</div></a>' % (link, image, name)


def plugin_linkcard(md):
    md.inline.register_rule('linkcard', LINKCARD_PATTERN, parse_linkcard)
    md.inline.rules.insert(0, 'linkcard')
    if md.renderer.NAME == 'html':
        md.renderer.register('linkcard', render_html_linkcard)



import mistune


class ParagraphRenderer(mistune.HTMLRenderer):
    def paragraph(self, text):
        # text contains figure tag
        if text.strip().startswith('<figure'):
            text = text.strip()
            if text[1:].find('<figure') > -1:
                # if there are at lease 2 figure tags
                text = [item.strip() for item in text.split('\n')]
                text = ['<div class="photos">%s</div>' % item for item in text]
                return '<div class="photoset">%s</div>' % ''.join(text)
            else:
                return text

        return '<p>%s</p>' % text



# -*- coding: utf-8 -*-
"""Configuration
"""

import os
import sys
from pathlib import Path

class Config(object):
    def update_from_file(self, filepath=None):
        if filepath is None:
            return

        sys.path.insert(0, os.path.dirname(filepath))

        name, _ = os.path.splitext(os.path.basename(filepath))
        module = __import__(name)

        attrs = dir(module)
        for attr in attrs:
            if attr.startswith('__'):
                continue
            setattr(self, attr, getattr(module, attr))

    def update_from_env(self):
        attrs = dir(self)
        for attr in attrs:
            if attr.startswith('__'):
                continue
            setattr(self, attr, os.getenv(attr, getattr(self, attr)))

    # For Maverick
    site_prefix = "/"
    mvrk_path = "."
    source_dir = mvrk_path + '/test_src/'
    build_dir = "./test_dist/"

    """Config theme for Maverick
    
    to use theme in another local folder, set:
    template = {
        "name": "<name of template, required>",
        "type": "local",
        "path": "<path to template, required>"
    }
    
    to use theme from a remote git repo, set:
    template = {
        "name": "<name of template, required>",
        "type": "git",
        "url": "<url of git repo, required>",
        "branch": "<branch of repo, optional, default to master>",
        "tag": "<tag of repo, optional, default to latest>"
    }
    """
    template = "Galileo"

    index_page_size = 10
    archives_page_size = 30
    fetch_remote_imgs = False
    enable_jsdelivr = {
        "enabled": False,
        "repo": ""
    }
    locale = "Asia/Shanghai"
    category_by_folder = False

    # !DEPRECIATE
    # This option will be removed in the future
    # prefer `output_image` hook and template specific config
    # to control rendering behavior of images
    parse_alt_as_figcaption = True

    # For site
    site_name = "Yet Another Maverick Site"
    site_logo = ""
    site_build_date = ""
    author = ""
    email = ""
    author_homepage = ""
    description = ""
    key_words = []
    language = "english"
    background_img = ""
    external_links = []
    nav = []

    social_links = []

    valine = {
        "enable": False,
    }

    head_addon = ""

    footer_addon = ""

    body_addon = ""


g_conf = Config()



# -*- coding: utf-8 -*-
"""Ruby
{{Ruby:ルビ}} -> <ruby>Ruby<rp>(</rp><rt>ルビ</rt><rp>)</rp></ruby>
"""

RUBY_PATTERN = (
    r'\{\{'                   # {{
    r'([\s\S]+?\:[\s\S]+?)'   # Text:Ruby
    r'\}\}(?!\})'             # }}
)


def parse_ruby(self, m, state):
    # ``self`` is ``md.inline``, see below
    # ``m`` is matched regex item
    text = m.group(1)
    text, ruby = text.split(':')
    return 'ruby', text, ruby


def render_html_ruby(text, ruby):
    return '<ruby>%s<rp>(</rp><rt>%s</rt><rp>)</rp></ruby>' % (text, ruby)


def plugin_ruby(md):
    md.inline.register_rule('ruby', RUBY_PATTERN, parse_ruby)
    md.inline.rules.append('ruby')
    if md.renderer.NAME == 'html':
        md.renderer.register('ruby', render_html_ruby)



# -*- coding: utf-8 -*-

import os
import re
import shutil
import codecs
import hashlib
import chardet
import json
import stat
import subprocess
from .Config import g_conf
from .Router import Router
from enum import Enum


class Color(Enum):
    BLACK = 30
    RED = 31
    GREEN = 32
    YELLOW = 33
    BLUE = 34
    MAGENTA = 35
    CYAN = 36
    WHITE = 37


def print_color(text: str, fg: Color = Color.BLACK.value, end='\n'):
    print('\033[%sm%s\033[0m' % (fg, text), end=end)

def run(command, cwd):
    with subprocess.Popen(command, cwd=cwd, shell=True) as ret:
        ret.wait()
        if ret.returncode != 0:
            ret.kill()
            raise BaseException

def logged_func(delim='\n'):
    def inner(func):
        def wrapper(*args, **kwargs):
            log_start(func.__name__, delim)
            func(*args, **kwargs)
            log_end()
        return wrapper
    return inner


def copytree(src, dst, syamlinks=False, ignore=None):
    for item in os.listdir(src):
        s = unify_joinpath(src, item)
        d = unify_joinpath(dst, item)
        if os.path.isdir(s):
            shutil.copytree(s, d, syamlinks, ignore)
        else:
            shutil.copy2(s, d)


def force_rmtree(top):
    for root, dirs, files in os.walk(top, topdown=False):
        for name in files:
            filename = os.path.join(root, name)
            os.chmod(filename, stat.S_IWUSR)
            os.remove(filename)
        for name in dirs:
            os.rmdir(os.path.join(root, name))
    os.rmdir(top)


def log_start(message, delim):
    print_color('Start ' + message, Color.YELLOW.value, end='...'+delim)


def log_end():
    print_color('done.', Color.YELLOW.value)


def safe_write(path, content, codec="utf-8"):
    if not os.path.exists(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))

    with codecs.open(path, "w+", encoding=codec) as f:
        f.write(content)


def safe_read(path):
    if not os.path.exists(path):
        return ""

    # utf-8 is prefered
    try:
        with open(path, 'r', encoding='utf-8') as f:
            return f.read()
    except UnicodeDecodeError:
        with open(path, 'rb') as f:
            content = f.read()
            encoding = chardet.detect(content)['encoding'] or 'utf-8'
            return content.decode(encoding=encoding)


def gen_hash(str):
    h1 = hashlib.md5()
    h1.update(str.encode(encoding='utf-8'))
    return h1.hexdigest()


def unify_joinpath(left, right):
    path = os.path.join(left, right)
    return path.replace('\\', '/')


def filterPlaceholders(content):
    """replace content like ${key} to corresponding value

    1. search key in env
    2. search key in config
    """
    pattern = re.compile(r'[\s\S]*?\$\{([\s\S]*?)\}')
    router = Router(g_conf)

    def getKey(str):
        m = pattern.match(str)
        if not m is None:
            return m.group(1)
        else:
            return None

    while True:
        key = getKey(content)
        if key is None:
            break

        search_str = '${%s}' % key
        value = ''
        if key == "static_prefix":
            value = router.gen_static_file_prefix()
        else:
            # find in os.env
            value = os.getenv(key, None)
            if value is None:
                # find in config
                try:
                    value = getattr(g_conf, key)
                except AttributeError:
                    pass

        # replace
        content = content.replace(search_str, str(value), 1)

    return content



# -*- coding: utf-8 -*-
"""Autotag
[tag][/tag] ==> <div class="tag"></div>
"""

import re

AUTOTAG_PATTERN = re.compile(
    r'\[(.*?)\]'  # [tag]
    r'([\s\S]+?)'
    r'\[/\1\]'       # [/tag]
)


def parse_autotag(self, m, state):
    tag, text = m.group(1), m.group(2)
    if tag.lower() == 'dplayer':
        return {'type': 'autotag', 'raw': text, 'params': (tag, )}
    elif tag.lower() == 'details':
        return {'type': 'autotag', 'text': text, 'params': (tag, )}
    else:
        return {'type': 'autotag', 'text': text, 'params': (tag, )}


def render_html_autotag(text, tag):
    if tag == 'dplayer':
        return '<div class="%s" data-url="%s"></div>' % (tag, text)
    elif tag == 'details':
        return '<p><details>%s</details></p>' % (text)
    return '<div class="%s">%s</div>' % (tag, text)


def plugin_autotag(md):
    md.block.register_rule('autotag', AUTOTAG_PATTERN, parse_autotag)
    md.block.rules.append('autotag')
    if md.renderer.NAME == 'html':
        md.renderer.register('autotag', render_html_autotag)



# -*- coding: utf-8 -*-
"""Math Block
$$ $$
"""

import re
import mistune

BLOCKMATH_PATTERN = re.compile(
    r'\$\$'         # $$
    r'([\s\S]+?)'
    r'\$\$'         # $$
)


def parse_mathblock(self, m, state):
    text = m.group(1)
    return {'type': 'mathblock', 'raw': mistune.escape(text)}


def render_html_mathblock(text):
    return '<p>$$%s$$</p>\n' % (text)


def plugin_mathblock(md):
    md.block.register_rule('mathblock', BLOCKMATH_PATTERN, parse_mathblock)
    md.block.rules.append('mathblock')
    if md.renderer.NAME == 'html':
        md.renderer.register('mathblock', render_html_mathblock)



# -*- coding: utf-8 -*-
import os
import sys
import click
import shutil

from .Maverick.Builder import Builder
from .Maverick.Config import g_conf

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
work_path = os.path.abspath('.')
root_path = os.path.abspath(os.path.dirname(__file__))
repo_default_dir_path = os.path.join(root_path, "default_dir")


@click.group(context_settings=CONTEXT_SETTINGS)
def repo_cli():
    pass


@repo_cli.command(name="create")
@click.argument("name")
def create(name):
    new_repo_path = os.path.join(work_path, name)
    click.echo(root_path)
    if os.path.exists(new_repo_path):
        click.echo("文件夹已经存在")
    else:
        shutil.copytree(repo_default_dir_path, new_repo_path)


@repo_cli.command(name="generate")
def generate():
    config_path = os.path.join(root_path, "Maverick/default_config.py")
    g_conf.update_from_file(config_path)
    click.echo(g_conf.mvrk_path)
    builder = Builder(g_conf)
    builder.build_all()


def main() -> None:
    repo_cli.main()


if __name__ == "__main__":
    main()




# -*- coding: utf-8 -*-
import urllib
import urllib.parse

class Router:
    def __init__(self, conf):
        """Create permalink for a content.
        """
        self.config = conf

    def gen_static_file_prefix(self):
        """Generate prefix for static file

        for files including:
            /assets/*
            /archives/assets/*
        
        return ${site_prefix} by default, however, if configured to use
        jsDelivr, then corresponding path will be used
        """
        if (self.config.enable_jsdelivr['enabled']):
            return "https://cdn.jsdelivr.net/gh/%s/" % \
                self.config.enable_jsdelivr["repo"]
        else:
            return self.config.site_prefix

    def gen_permalink_by_meta(self, meta):
        link, _ = self.gen_by_meta(meta)
        return link

    def gen_permalink_by_content(self, content):
        link, _ = self.gen_by_content(content)
        return link

    def gen_permalink(self, type, slug, page=0):
        link, _ = self.gen(type, slug, page)
        return link

    def gen_by_meta(self, meta):
        return self.gen(meta["layout"], meta["slug"])

    def gen_by_content(self, content):
        return self.gen(content.get_meta('layout'), content.get_meta('slug'))

    def gen(self, type, slug, page=1):
        routes = list()
        if type == "post":
            routes = ["archives", slug]
        elif type == "page":
            routes = [slug]
        elif type == "tag" or type == "category":
            routes = [type, slug]
        elif type == "archives":
            routes = ["archives"]
        elif type == "index" and page > 1:
            routes = ["page"]

        if page > 1:
            routes.append(str(page))

        path = "/".join(routes)
        if len(routes):
            path += "/"

        local_path = self.config.build_dir + path
        permalink = self.config.site_prefix + urllib.parse.quote(path)

        return permalink, local_path



# -*- coding: utf-8 -*-
"""Inline Footnote
text[^footnote]
"""

import re
from ..Utils import gen_hash


INLINE_FOOTNOTE_PATTERN = (
    r'\[\^'             # [^
    r'([\s\S]*?)'       # text
    r'\]'               # ]
)


def parse_inlinefootnote(self, m, state):
    match = re.match(INLINE_FOOTNOTE_PATTERN, m.group(0))
    text = match.group(1)
    index = state.get('footnote_index', 0) + 1
    
    if text in state['def_footnotes']:
        # text is the key for a def_footnote
        state['footnotes'].append(text)
    else:
        # test is content for a inline footnote
        hash = gen_hash(text)[:5]
        state['def_footnotes'][hash] = text
        state['footnotes'].append(hash)

    state['footnote_index'] = index
    return 'inlinefootnote', str(index), text


def render_html_inlinefootnote(index, text):
    html = '<sup class="footnote-ref" id="fnref-' + index + '">'
    return html + '<a href="#fn-' + index + '">' + index + '</a></sup>'


def plugin_inlinefootnote(md):
    md.inline.register_rule(
        'inlinefootnote', INLINE_FOOTNOTE_PATTERN, parse_inlinefootnote)
    index = md.inline.rules.index('footnote')
    if index != -1:
        md.inline.rules.insert(index, 'inlinefootnote')
    else:
        md.inline.rules.append('inlinefootnote')
    if md.renderer.NAME == 'html':
        md.renderer.register('inlinefootnote', render_html_inlinefootnote)



# -*- coding: utf-8 -*-

from .Metadata import Metadata
from .Markdown import Markdown
from .Utils import safe_read
from .Config import g_conf

import re
import yaml


def group_by_tagname(tag):
    def criteria(content):
        return tag in content.get_meta('tags')
    return criteria


def group_by_category(category):
    def criteria(content):
        # direct category
        return category == content.get_meta('categories')[-1]
    return criteria


class Content():
    def __init__(self, path):
        content = safe_read(path)

        # 获取文章源部分
        r = re.compile(r'---([\s\S]*?)---')
        m = r.match(content)

        # 获取文章源信息
        self.meta = Metadata(yaml.safe_load(m.group(1)))
        self.meta['path'] = path
        self.text = content[len(m.group(0)):]

        self.skip = not self.get_meta('status').lower() in [
            "publish", "published"]

        self._parsed = None
        self._excerpt = None

    @property
    def excerpt(self):
        if self._excerpt is None:
            def strip(text):
                    r = re.compile(r'<[^>]+>', re.S)
                    return r.sub('', text)

            output = self.get_meta("excerpt")
            if output != "" and output != "None":
                return output

            # find <!--more-->
            index = self.parsed.find('<!--more-->')
            if index != -1:
                output = strip(self.parsed[:index])
            else:
                output = strip(self.parsed)
                output = output[:output.find('\n')]
            self._excerpt = output

        return self._excerpt

    @property
    def parsed(self):
        if self._parsed is None:
            self._parsed = Markdown(self)
        return self._parsed

    def get_meta(self, key, default=None):
        return self.meta.get(key, default)

    def update_meta(self, key, value):
        self.meta[key] = value

    @staticmethod
    def cmp_by_date(content_1, content_2):
        date_1 = content_1.get_meta('date')
        date_2 = content_2.get_meta('date')
        if date_1 < date_2:
            return -1
        elif date_1 > date_2:
            return 1
        else:
            return 0


class ContentList(list):
    """Content List.

    Convinient for sorting and slicing.
    """

    def __init__(self, from_obj=[]):
        list.__init__(self, from_obj)

    def re_group(self, _func):
        """subset items with _func(item) = True
        """
        resultList = ContentList()
        for meta in self:
            if _func(meta) == True:
                resultList.append(meta)
        return resultList



import mistune
from pygments import highlight
from pygments.lexers import get_lexer_by_name
from pygments.formatters import html


class HighlightRenderer(mistune.HTMLRenderer):
    def fallback(self, code, lang=None):
        return '\n<pre><code>%s</code></pre>\n' % \
            mistune.escape(code)
    
    def block_code(self, code, lang=None):

        if lang:
            try:
                lexer = get_lexer_by_name(lang, stripall=True)
                formatter = html.HtmlFormatter()
                return highlight(code, lexer, formatter)
            except:
                return self.fallback(code, lang)
        else:
            return self.fallback(code, lang)



# -*- coding: utf-8 -*-
"""Tweaked Mistune
"""

import os
import re
from .Cache import cache_img
import mistune
from mistune.plugins import plugin_strikethrough, plugin_footnotes, plugin_table, plugin_url
from .mistune_plugins import plugin_ruby, plugin_autotag, plugin_linkcard, plugin_inlinefootnote
from .mistune_plugins import plugin_mathblock
from .mistune_plugins import MyRenderer


class ParserHook(dict):
    """Save hook as key-value pairW
    """

    def add_hook(self, hook: str, callback: callable):
        self[hook] = callback

    def remove_hook(self, hook: str):
        if hook in self.keys():
            self.pop(hook)


g_hooks = ParserHook()


def Markdown(content):
    renderer = MyRenderer(
        escape=False, md_path=content.get_meta("path"), g_hooks=g_hooks)
    markdown = mistune.Markdown(renderer, plugins=[
        plugin_table, plugin_footnotes, plugin_strikethrough, plugin_url,
        plugin_autotag, plugin_mathblock,
        plugin_linkcard, plugin_ruby, plugin_inlinefootnote
    ])
    return markdown(content.text)



# -*- coding: utf-8 -*-
"""Handle cache
"""

import os
import json
import shutil
from PIL import Image
from PIL import ImageFile
import urllib
from .Config import g_conf
from .Router import Router

from urllib.parse import urlparse
import urllib.request as request
from .Utils import unify_joinpath, print_color, Color, safe_read, safe_write, gen_hash

g_used_imgs = None
g_sizeinfo_cache = None


def quickQueryImgSize(src):
    """get size info by downloading small part of it
    """
    try:
        with request.urlopen(src) as file:
            p = ImageFile.Parser()
            while True:
                data = file.read(1024)
                if not data:
                    break
                p.feed(data)
                if p.image:
                    return list(p.image.size)
    except BaseException as e:
        print_color('fetch error: ' + src, Color.RED.value)
        print_color(e, Color.RED.value)
        return None


def cache_img(src, base_path):
    """Gen image cache

    1. find image in cache dir
    2. if src is remote URL, download it or call quickQueryImgSize
    3. treat src as absolute path and find it
    3. treat src as relative path and find it
    """
    global g_used_imgs
    global g_sizeinfo_cache

    json_path = unify_joinpath(g_conf.mvrk_path, 'tmp/used_imgs.json')
    sizeinfo_json_path = unify_joinpath(g_conf.mvrk_path, 'cached_imgs/sizeinfo.json')
    cache_dir = unify_joinpath(g_conf.mvrk_path, 'cached_imgs')

    if g_used_imgs is None:
        g_used_imgs = set(json.loads(safe_read(json_path) or '[]'))
    if g_sizeinfo_cache is None:
        g_sizeinfo_cache = json.loads(
            safe_read(sizeinfo_json_path) or '{}')

    def log_and_return(filename):
        global g_used_imgs
        global g_sizeinfo_cache

        g_used_imgs = g_used_imgs | set([filename])

        info = {
            "src": '',
            "width": -1,
            "height": -1
        }

        # if enable jsDelivr CDN, add prefix
        # if not, fallback (site_prefix) will be used
        router = Router(g_conf)
        static_prefix = router.gen_static_file_prefix()
        info['src'] = "%sarchives/assets/%s" % (static_prefix, filename)

        if filename in g_sizeinfo_cache:  # if size info in cache
            info['width'] = g_sizeinfo_cache[filename][0]
            info['height'] = g_sizeinfo_cache[filename][1]
            print_color("Sizeinfo hit cache: %s (%s, %s)" %
                        (src, info['width'], info['height']), Color.GREEN.value)
        else:
            try:
                img = Image.open(unify_joinpath(cache_dir, filename))
                info['width'] = img.size[0]
                info['height'] = img.size[1]
                print_color("Parsed sizeinfo from local: %s (%s, %s)" %
                            (src, info['width'], info['height']), Color.GREEN.value)
                g_sizeinfo_cache[filename] = [img.size[0], img.size[1]]
            except IOError:
                print_color("Pars sizeinfo from local failed", Color.RED.value)

        return info

    src_md5 = gen_hash(src)

    # find image in cache dir
    if not os.path.exists(cache_dir):
        os.makedirs(cache_dir)
    cached_imgs = [name for name in os.listdir(cache_dir)]
    for name in cached_imgs:
        if name.split('.')[0].lower() == src_md5.lower():  # in cache dir
            # print_color("Image hit cache: " + src, Color.GREEN.value)
            return log_and_return(name)

    # if it is remote image
    if src.startswith('http'):
        if g_conf.fetch_remote_imgs:
            # download and treat it as local image
            try:
                suffix = urlparse(src).path.split('.')[-1]
                filename = '.'.join([src_md5, suffix])

                proxy = request.ProxyHandler({})
                opener = request.build_opener(proxy)
                opener.addheaders = [('User-Agent',
                                      r'Mozilla/5.0 (Windows NT 10.0; Win64; x64) '
                                      r'AppleWebKit/537.36(KHTML, like Gecko) '
                                      r'Chrome/78.0.3904.108 Safari/537.36')]
                request.install_opener(opener)

                print_color("Trying to download: " + src, Color.BLUE.value)
                request.urlretrieve(
                    src, unify_joinpath(cache_dir, filename))

                return log_and_return(filename)
            except BaseException as e:
                print_color('Fetch error: ' + src, Color.RED.value)
                print_color(e, Color.RED.value)
                return {
                    "src": src,
                    "width": -1,
                    "height": -1
                }
        else:
            # download small part of it
            info = {
                "src": src,
                "width": -1,
                "height": -1
            }

            if src in g_sizeinfo_cache:
                info['width'] = g_sizeinfo_cache[src][0]
                info['height'] = g_sizeinfo_cache[src][1]
                print_color("Sizeinfo hit cache: %s (%s, %s)" %
                            (src, info['width'], info['height']), Color.GREEN.value)
            else:
                print_color("Trying to fetch size of "+src, Color.BLUE.value)
                size = quickQueryImgSize(src)
                if not size is None:
                    info['width'] = size[0]
                    info['height'] = size[1]
                    g_sizeinfo_cache[src] = size
                    print_color("Size fetched: (%s, %s)" % (
                        info['width'], info['height']), Color.BLUE.value)

            return info

    # treat src as absolute path
    if os.path.exists(src):
        print_color("Image found at local: " + src, Color.GREEN.value)
        filename = src_md5 + '.' + src.split('.')[-1]
        shutil.copyfile(src, unify_joinpath(cache_dir, filename))
        return log_and_return(filename)

    # treat src as relative path to Markdown file
    if os.path.exists(unify_joinpath(base_path, src)):
        print_color("Image found at local: " + src, Color.GREEN.value)
        filename = src_md5 + '.' + src.split('.')[-1]
        shutil.copyfile(unify_joinpath(base_path, src),
                        unify_joinpath(cache_dir, filename))
        return log_and_return(filename)

    return {
        "src": src,
        "width": -1,
        "height": -1
    }


def dump_log():
    global g_used_imgs
    global g_sizeinfo_cache

    json_path = unify_joinpath(g_conf.mvrk_path, 'tmp/used_imgs.json')
    sizeinfo_json_path = unify_joinpath(g_conf.mvrk_path, 'cached_imgs/sizeinfo.json')

    safe_write(json_path, json.dumps(list(g_used_imgs or []), indent=1))
    safe_write(sizeinfo_json_path, json.dumps(
        g_sizeinfo_cache or {}, indent=1))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['repo_cli',
 'repo_cli.Maverick',
 'repo_cli.Maverick.Galileo',
 'repo_cli.Maverick.mistune_plugins']

package_data = \
{'': ['*'],
 'repo_cli': ['default_dir/*',
              'default_dir/test_src/*',
              'default_dir/test_src/assets/*',
              'default_dir/test_src/static/*',
              'default_dir/test_src/static/bg/*'],
 'repo_cli.Maverick.Galileo': ['assets/*',
                               'assets/ExSearch/*',
                               'assets/fonts/*',
                               'locale/*',
                               'templates/*',
                               'templates/includes/*']}

install_requires = \
['Jinja2>=3.0.1,<4.0.0',
 'Pillow>=8.2.0,<9.0.0',
 'PyYAML>=5.4.1,<6.0.0',
 'Pygments>=2.9.0,<3.0.0',
 'chardet>=4.0.0,<5.0.0',
 'click>=7.0,<8.0',
 'feedgen>=0.9.0,<0.10.0',
 'mistune==2.0.0a4',
 'moment>=0.12.1,<0.13.0']

entry_points = \
{'console_scripts': ['repo-cli = repo_cli.entry:main']}

setup_kwargs = {
    'name': 'repo-cli',
    'version': '0.0.1.14',
    'description': 'standard tooling for repo project.',
    'long_description': '# repo-cli\nstandard tooling for repo project.\n\n## note\n\nThe project is under development, please do not install and use it for now.',
    'author': 'RyomaHan',
    'author_email': 'ryomahan1996@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ryomahan/repo-cli',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
"""Utils for Galileo
"""

import os
import json
from ..Config import g_conf
from ..Utils import unify_joinpath, safe_read, filterPlaceholders

translation = None


def tr(str, locale="english"):
    """translation support

    translate str according to translation file
    """
    global translation
    if translation is None:
        path = unify_joinpath(os.path.dirname(
            __file__) + '/locale', g_conf.language+".json")
        translation = json.loads(safe_read(path) or '{}')

    return translation.get(str, str)


def build_links(links):
    fp = filterPlaceholders
    str = '<span class="separator">·</span>'.join(['<li><a class="no-style" title="%s" href="%s" target="_blank"><i class="%s"></i>%s</a></li>'
                                                   % (fp(item['name']), fp(item['url']), fp(item['icon']), fp(item['name'])) for item in links])
    return '<ul>%s</ul>' % str


def build_navs(navs):
    fp = filterPlaceholders
    list = ['<li><a class="ga-highlight" href="%s" target="%s">%s</a></li>'
            % (fp(item['url']), fp(item['target']), fp(item['name'])) for item in navs]
    list.append(
        '<li><a href="#" target="_self" class="search-form-input ga-highlight">%s</a></li>' % tr('Search'))
    return '<ul>%s</ul>' % ('<span class="separator">·</span>'.join(list))


def filterPrefix(url: str):
    """replace prefix with `/`, to fix Valine view counting
    """
    return url.replace(g_conf.site_prefix, "/")
